#ifndef _profile_matrix_c /* Copyright Bill.Triggs@inrialpes.fr */
#define _profile_matrix_c "$ Sun Jan 18 17:44:33 2004 $"
/** NO WARRANTY. Use & distribute freely. Fixes/feedback welcome. **/
/*------------------------------------------------------------------------
  Memory allocation for a matrix in profile / skyline representation.
  We actually work by rows not columns, i.e. the matrix is a vector
  (with indices 0..nrows-1) of pointers to a series of rows, each of
  which contains a single contiguous block of cells with indices
  cmin[i]..cmax[i]. If explicit cmin or cmax vectors are not supplied,
  cmin,cmax[i]=i is assumed, i.e. indices start or end at the
  diagonal.

  The pointer vector and cells are allocated as a single contiguous
  block of memory.  We pass the matrix element size to the allocator,
  so that the same allocator can be used for different element types
  (float, double, int).

 ------------------------------------------------------------------------*/
#ifndef _profile_matrix_h
# include "profile_matrix.h"
#endif

/* Memory allocator to use, by default malloc/free.
 */
#ifndef ALLOC
# include <stdlib.h>
# define ALLOC(type,size)	(type*)malloc((size)*sizeof(type))
# define FREE free
#endif

/* Round up size of a chunk of memory to retain good memory alignment.
 */
#ifndef MEMALIGN
# define MEMALIGN(x)	((((x)+ALIGNSIZE-1)/ALIGNSIZE)*ALIGNSIZE)
# ifndef ALIGNSIZE
#  define ALIGNSIZE	(sizeof(double))
# endif
#endif

/* Dummy pointer type to use for matrix pointer vector.
 */
#ifndef ptr_t
# define ptr_t void*
#endif

/*--------------------------------------------------------------------
  Profile matrix allocator. 

  elsize - sizeof matrix element (so that same allocator can be used
  for different element types).

  nrows - matrix row indices are 0..nrows-1

  cmin,cmax - matrix column indices are cmin[i]..cmax[i] in row i. If
  cmin or cmax is not supplied, i (i.e. matrix diagonal) is assumed.

 --------------------------------------------------------------------*/
void*
_new_profile_matrix(int elsize, int nrows, const int cmin[], const int cmax[])
{
  unsigned long ptrsize = MEMALIGN(nrows*sizeof(ptr_t)), ncells = 0;
  ptr_t* p;
  char* c;
  int i;
  /* Count number of matrix cells to be allocated.
   */
  if (cmin && cmax) {
     for (i=0; i<nrows; i++) ncells += cmax[i]-cmin[i]+1;
  } else if (cmin) {
     for (i=0; i<nrows; i++) ncells += i-cmin[i]+1;
  } else if (cmax) {
     for (i=0; i<nrows; i++) ncells += cmax[i]-i+1;
  } else {
     ncells = nrows;
  }
  /* Allocate pointer and element vector.
   */
  p = (ptr_t*)ALLOC(char,ptrsize+ncells*elsize);
  c = (char*)p + ptrsize;

  /* Wire up row pointers with correct offsets so that elements
   * p[i][j] for j in cmin[i]..cmax[i] are allocated.
   */
  if (cmin && cmax) {
     for (i=0; i<nrows; i++) { p[i] = (ptr_t)(c-=cmin[i]*elsize); c += (cmax[i]+1)*elsize; }
  } else if (cmin) {
     for (i=0; i<nrows; i++) { p[i] = (ptr_t)(c-=cmin[i]*elsize); c += (i+1)*elsize; }
  } else if (cmax) {
     for (i=0; i<nrows; i++) { p[i] = (ptr_t)(c-=i*elsize); c += (cmax[i]+1)*elsize; }
  } else {
     for (i=0; i<nrows; i++) { p[i] = (ptr_t)c; }
  }
  return (void*)p;
}

void
free_profile_matrix(void* mat)
{
   if (mat) 
      FREE(mat);
}

#endif /*_profile_matrix_c */
